/*
 * Quick & Dirty Debug Register Read Module for ARM devices
 *
 * Copyright (C) 2012 Johannes Goetzfried <johannes@jgoetzfried.de>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307 USA.
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/err.h>
#include <linux/smp.h>


static void print_debug_current_cpu(void *dummy)
{
	u32 b0 = 0;
	u32 b1 = 0;
	u32 b2 = 0;
	u32 b3 = 0;
	u32 w0 = 0;
	u32 w1 = 0;
	u32 w2 = 0;
	u32 w3 = 0;

	__asm__(
		"mrc p14, 0, %[br0], c0, c0, 4\n\t"
		"mrc p14, 0, %[br1], c0, c1, 4\n\t"
		"mrc p14, 0, %[br2], c0, c2, 4\n\t"
		"mrc p14, 0, %[br3], c0, c3, 4\n\t"
		: [br0] "=r" (b0),
		  [br1] "=r" (b1),
		  [br2] "=r" (b2),
		  [br3] "=r" (b3) ::
	);

	__asm__(
		"mrc p14, 0, %[wp0], c0, c0, 6\n\t"
		"mrc p14, 0, %[wp1], c0, c1, 6\n\t"
		"mrc p14, 0, %[wp2], c0, c2, 6\n\t"
		"mrc p14, 0, %[wp3], c0, c3, 6\n\t"
		: [wp0] "=r" (w0),
		  [wp1] "=r" (w1),
		  [wp2] "=r" (w2),
		  [wp3] "=r" (w3) ::
	);

	printk(KERN_INFO "\nRegisters on CPU %d\n", smp_processor_id());
	printk(KERN_INFO "br0: 0x%08x\n", b0);
	printk(KERN_INFO "br1: 0x%08x\n", b1);
	printk(KERN_INFO "br2: 0x%08x\n", b2);
	printk(KERN_INFO "br3: 0x%08x\n", b3);
	printk(KERN_INFO "wp0: 0x%08x\n", w0);
	printk(KERN_INFO "wp1: 0x%08x\n", w1);
	printk(KERN_INFO "wp2: 0x%08x\n", w2);
	printk(KERN_INFO "wp3: 0x%08x\n", w3);
}


static int __init debug_init(void)
{
	on_each_cpu(print_debug_current_cpu, (void *)NULL, 1);

	return -ENOMEM;
}

static void __exit debug_exit(void)
{
}

module_init(debug_init);
module_exit(debug_exit);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Quick & Dirty Debug Register Read Module");
MODULE_AUTHOR("Johannes Goetzfried");
